#############################################################
#
# sablevm
#
#############################################################

SABLEVM_VER:=1.13
SABLEVM_DIR:=$(BUILD_DIR)/sablevm-$(SABLEVM_VER)
SABLEVM_SITE:=http://sablevm.org/download/release/$(SABLEVM_VER)/
SABLEVM_SOURCE:=sablevm-$(SABLEVM_VER).tar.gz
SABLEVM_CAT:=zcat
SABLEVM_LIBS_DIR:=$(SABLEVM_DIR)/src/libsablevm/.libs

$(DL_DIR)/$(SABLEVM_SOURCE):
	 $(WGET) -P $(DL_DIR) $(SABLEVM_SITE)/$(SABLEVM_SOURCE)

sablevm-source: $(DL_DIR)/$(SABLEVM_SOURCE)

$(SABLEVM_DIR)/.unpacked: $(DL_DIR)/$(SABLEVM_SOURCE)
	$(SABLEVM_CAT) $(DL_DIR)/$(SABLEVM_SOURCE) | tar -C $(BUILD_DIR) $(TAR_OPTIONS) -
	toolchain/patch-kernel.sh $(SABLEVM_DIR) package/sablevm/ sablevm\*.patch
	touch $(SABLEVM_DIR)/.unpacked

sablevm-unpack: $(SABLEVM_DIR)/.unpacked

$(SABLEVM_DIR)/.configured: $(SABLEVM_DIR)/.unpacked
	(cd $(SABLEVM_DIR); rm -rf config.cache; \
		$(TARGET_CONFIGURE_OPTS) \
		LD=$(TARGET_CROSS)ld \
		CFLAGS="$(TARGET_CFLAGS) -O3" \
		LDFLAGS="-L$(STAGING_DIR)/lib -L$(STAGING_DIR)/usr/lib" \
		./configure \
		--target=$(GNU_TARGET_NAME) \
		--host=$(GNU_TARGET_NAME) \
		--build=$(GNU_HOST_NAME) \
		--prefix=/usr \
		--exec-prefix=/usr \
		--bindir=/usr/bin \
		--sbindir=/usr/sbin \
		--libdir=/lib \
		--libexecdir=/usr/lib \
		--sysconfdir=/etc \
		--datadir=/usr/share \
		--localstatedir=/var \
		--mandir=/usr/man \
		--includedir=$(STAGING_DIR)/include \
		--infodir=/usr/info \
		--disable-libtool \
		$(DISABLE_NLS) \
		$(DISABLE_LARGEFILE) \
		--with-internal-libffi=yes \
		--with-internal-libpopt=yes \
	);
	touch $(SABLEVM_DIR)/.configured

$(SABLEVM_DIR)/.compiled: $(SABLEVM_DIR)/.configured
	$(MAKE) -C $(SABLEVM_DIR)
	touch $(SABLEVM_DIR)/.compiled

$(SABLEVM_LIBS_DIR)/libsablevm.so: $(SABLEVM_DIR)/.compiled
	touch -c $@

$(STAGING_DIR)/lib/libsablevm.so: $(SABLEVM_LIBS_DIR)/libsablevm.so
	mkdir -p $(STAGING_DIR)/lib
	cp -dpf $(SABLEVM_LIBS_DIR)/libsablevm*.so $(STAGING_DIR)/lib/
	-$(STRIP) --strip-unneeded $(STAGING_DIR)/lib/libsablevm*.so

$(TARGET_DIR)/usr/lib/libsablevm.so: $(STAGING_DIR)/lib/libsablevm.so
	mkdir -p $(TARGET_DIR)/usr/lib
	cp -dpf $(STAGING_DIR)/lib/libsablevm*.so $(TARGET_DIR)/usr/lib/

sablevm: $(TARGET_DIR)/usr/lib/libsablevm.so

sablevm-clean:
	-$(MAKE) -C $(SABLEVM_DIR) clean
	-rm -f $(TARGET_DIR)/usr/lib/libsablevm.so*
	-rm -f $(STAGING_DIR)/lib/libsablevm.so*
	-rm -f $(SABLEVM_DIR)/.configured $(SABLEVM_DIR)/.compiled

sablevm-dirclean:
	rm -rf $(SABLEVM_DIR)

#############################################################
#
# Toplevel Makefile options
#
#############################################################
ifeq ($(strip $(BR2_PACKAGE_SABLEVM)),y)
TARGETS+=sablevm
endif
