#include <stdlib.h>
#include <string.h>
#include "str.h"
#include "common.h"
#include "log.h"

cp_string *cp_string_create(char *data, int len)
{
	cp_string *str = calloc(1, sizeof(cp_string));

	if (str)
	{
		str->len = len;
		str->data = malloc((str->len + 1) * sizeof(char));
		if (str->data)
			memcpy(str->data, data, (str->len + 1) * sizeof(char));
		else
		{
			cp_error(CP_MEMORY_ALLOCATION_FAILURE, "can\'t create string");
			free(str);
			str = NULL;
		}
	}

	return str;
}

void cp_string_delete(cp_string *str)
{
	if (str)
	{
		if (str->data)
			free(str->data);

		free(str);
	}
}
	
cp_string *cp_string_cstrcpy(char *cstr)
{
	cp_string *str = calloc(1, sizeof(cp_string));
	
	if (str)
	{
		str->len = strlen(cstr);
		str->data = malloc((str->len + 1) * sizeof(char));
		if (str->data)
			memcpy(str->data, cstr, (str->len + 1) * sizeof(char));
		else
		{
			free(str);
			str = NULL;
		}
	}

	return str;
}

cp_string *cp_string_cpy(cp_string *src)
{
	cp_string *str = calloc(1, sizeof(cp_string));

	if (str)
	{
		*str = *src; /* bitwise copy */
		str->data = malloc((str->len + 1) * sizeof(char));
		if (str->data)
			memcpy(str->data, src->data, (str->len  + 1) * sizeof(char));
		else
		{
			free(str);
			str = NULL;
		}
	}

	return str;
}

cp_string *cp_string_cat(cp_string *str, cp_string *appendum)
{
	int len = str->len;
	str->len += appendum->len;
	str->data = realloc(str->data, (str->len + 1) * sizeof(char));
	if (str->data)
		memcpy(str->data + len * sizeof(char), appendum->data, 
			appendum->len * sizeof(char));

	return str;
}

cp_string *cp_string_cstrcat(cp_string *str, char *cstr)
{
	int len = str->len;
	int clen = strlen(cstr);

	str->len += clen * sizeof(char);
	if (str->len > str->size)
	{
		str->size = (str->len + 0x400) % 0x400; /* align to 1kb block */
		str->data = realloc(str->data, str->size * sizeof(char));
	}
	if (str->data)
		memcpy(str->data + len * sizeof(char), cstr, clen);

	return str;
}

cp_string *cp_string_cat_bin(cp_string *str, char *bin, int len)
{
	int olen = str->len;
	str->len += len;

	if (str->len > str->size)
	{
		int diff = str->len % 0x400;
		str->size = str->len;
		if (diff) str->size += (0x400 - diff); /* align to 1kb block */
		str->data = realloc(str->data, str->size * sizeof(char));
	}
	memcpy(&str->data[olen], bin, len);

	return str;
}

int cp_string_cmp(cp_string *s1, cp_string *s2)
{
	if (s1 == s2) return 0; //~~ implies cp_string_cmp(NULL, NULL) == 0

	if (s1 == NULL || s2 == NULL) return -1; //~~ ?

	{
		int p;
		if ((p = s1->len - s2->len)) return p;
	}
	
	return memcmp(s1->data, s2->data, s1->len * sizeof(char));
}

char *cp_string_tocstr(cp_string *str)
{
	char *cstr = NULL;
	
	if (str)
	{
		str->data[str->len * sizeof(char) + 1] = '\0';
		cstr = str->data;
	}

	return cstr;
}

